@@ -1,5 +1,6 @@
 
 import gzip
+import os
 
 import data_algebra
 import data_algebra.data_ops
@@ -37,12 +38,29 @@ def _bigquery_mean_expr(dbmodel, expression):
 def _bigquery_size_expr(dbmodel, expression):
     return "SUM(1)"
 
+def _bigquery_is_bad_expr(dbmodel, expression):
+    subexpr = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
+    return (
+        "("
+        + subexpr
+        + " IS NULL OR "
+        + "IS_INF(" + subexpr + ")"
+        + " OR ("
+        + subexpr
+        + " != 0 AND "
+        + subexpr
+        + " = -"
+        + subexpr
+        + "))"
+    )
+
 
 BigQuery_formatters = {
     "nunique": _bigquery_nunique_expr,
     "median": _bigquery_median_expr,
     "mean": _bigquery_mean_expr,
     "size": _bigquery_size_expr,
+    "is_bad": _bigquery_is_bad_expr,
 }
 
 
@@ -50,7 +68,7 @@ class BigQueryModel(data_algebra.db_model.DBModel):
     """A model of how SQL should be generated for BigQuery
        connection should be google.cloud.bigquery.client.Client"""
 
-    def __init__(self):
+    def __init__(self, *, table_prefix=None):
         data_algebra.db_model.DBModel.__init__(
             self,
             identifier_quote='`',
@@ -61,14 +79,24 @@ class BigQueryModel(data_algebra.db_model.DBModel):
             on_joiner=' AND ',
             string_type='STRING',
         )
+        self.table_prefix = table_prefix
 
-    def quote_identifier(self, identifier):
-        if not isinstance(identifier, str):
-            raise TypeError("expected identifier to be a str")
-        if self.identifier_quote in identifier:
-            # TODO: escape quotes
-            raise ValueError('did not expect ' + self.identifier_quote + ' in identifier')
-        return self.identifier_quote + identifier + self.identifier_quote
+    def quote_table_name(self, table_description):
+        if not isinstance(table_description, str):
+            try:
+                if table_description.node_name == "TableDescription":
+                    table_description = table_description.table_name
+                else:
+                    raise TypeError(
+                        "Expected table_description to be a string or data_algebra.data_ops.TableDescription)"
+                    )
+            except KeyError:
+                raise TypeError(
+                    "Expected table_description to be a string or data_algebra.data_ops.TableDescription)"
+                )
+        if self.table_prefix is not None:
+            table_description = self.table_prefix + '.' + table_description
+        return self.quote_identifier(table_description)
 
     # noinspection PyMethodMayBeStatic
     def execute(self, conn, q):
@@ -104,6 +132,28 @@ class BigQueryModel(data_algebra.db_model.DBModel):
         r.reset_index(drop=True, inplace=True)
         return r.copy()  # fresh copy
 
+    def insert_table(
+        self, conn, d, table_name, *, qualifiers=None, allow_overwrite=False
+    ):
+        prepped_table_name = table_name
+        if self.table_prefix is not None:
+            prepped_table_name = self.table_prefix + '.' + table_name
+        if allow_overwrite:
+            self.drop_table(conn, table_name)
+        else:
+            table_exists = True
+            try:
+                self.read_query(conn, "SELECT * FROM " + self.quote_table_name(table_name) + " LIMIT 1")
+                table_exists = True
+            except Exception as e:
+                table_exists = False
+            if table_exists:
+                raise ValueError("table " + prepped_table_name + " already exists")
+        job = conn.load_table_from_dataframe(
+            d,
+            prepped_table_name)
+        job.result()
+
     def db_handle(self, conn):
         return BigQuery_DBHandle(db_model=self, conn=conn)
 
@@ -160,22 +210,20 @@ class BigQuery_DBHandle(data_algebra.db_model.DBHandle):
                 block.to_csv(res, index=False, header=is_first)
                 is_first = False
 
-    def drop_table(self, table_name):
-        self.execute(f'DROP TABLE IF EXISTS `{table_name}`')
 
-    def insert_table(self, d, *, table_name, allow_overwrite=False):
-        if allow_overwrite:
-            self.drop_table(table_name)
-        else:
-            table_exists = True
-            try:
-                self.read_query("SELECT * FROM " + table_name + " LIMIT 1")
-                table_exists = True
-            except Exception:
-                table_exists = False
-            if table_exists:
-                raise ValueError("table " + table_name + " already exists")
-        job = self.conn.load_table_from_dataframe(
-            d,
-            table_name)
-        job.result()
+def example_handle():
+    """
+    Return an example db handle for testing. Returns None if helper packages not present.
+
+    """
+    # TODO: parameterize this
+    if not _have_bigquery:
+        return None
+    os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = "/Users/johnmount/big_query/big_query_jm.json"
+    # os.environ["GOOGLE_APPLICATION_CREDENTIALS"]  # trigger key error if not present
+    data_catalog = 'data-algebra-test'
+    data_schema = 'test_1'
+    db_handle = BigQueryModel(
+        table_prefix=f'{data_catalog}.{data_schema}').db_handle(google.cloud.bigquery.Client())
+    db_handle.db_model.prepare_connection(db_handle.conn)
+    return db_handle
