@@ -5,6 +5,8 @@ import io
 from types import SimpleNamespace
 from collections import OrderedDict
 
+import pandas.io.sql
+
 import data_algebra
 
 import data_algebra.near_sql
@@ -259,6 +261,7 @@ class DBModel:
         self.local_data_model = local_data_model
         if sql_formatters is None:
             sql_formatters = {}
+        assert identifier_quote != string_quote
         self.identifier_quote = identifier_quote
         self.string_quote = string_quote
         self.sql_formatters = sql_formatters.copy()
@@ -286,53 +289,6 @@ class DBModel:
 
     # database helpers
 
-    # noinspection SqlNoDataSourceInspection
-    def insert_table(
-        self, conn, d, table_name, *, qualifiers=None, allow_overwrite=False
-    ):
-        """
-
-        :param conn: a database connection
-        :param d: a Pandas table
-        :param table_name: name to give write to
-        :param qualifiers: schema and such
-        :param allow_overwrite logical, if True drop previous table
-        """
-
-        cr = [
-            d.columns[i].lower()
-            + " "
-            + (
-                "double precision"
-                if self.local_data_model.can_convert_col_to_numeric(d[d.columns[i]])
-                else self.string_type
-            )
-            for i in range(d.shape[1])
-        ]
-        q_table_name = self.quote_table_name(
-            table_name
-        )
-        cur = conn.cursor()
-        # check for table
-        table_exists = True
-        # noinspection PyBroadException
-        try:
-            self.read_query(conn, "SELECT * FROM " + q_table_name + " LIMIT 1")
-        except Exception:
-            table_exists = False
-        if table_exists:
-            if not allow_overwrite:
-                raise ValueError("table " + q_table_name + " already exists")
-            else:
-                cur.execute(self.drop_text + ' ' + q_table_name)
-                conn.commit()
-        create_stmt = "CREATE TABLE " + q_table_name + " ( " + ", ".join(cr) + " )"
-        cur.execute(create_stmt)
-        conn.commit()
-        buf = io.StringIO(d.to_csv(index=False, header=False, sep="\t"))
-        cur.copy_from(buf, "d", columns=[c for c in d.columns])
-        conn.commit()
-
     # noinspection PyMethodMayBeStatic
     def execute(self, conn, q):
         """
@@ -344,8 +300,7 @@ class DBModel:
             q = q.to_sql(db_model=self)
         else:
             q = str(q)
-        cur = conn.cursor()
-        cur.execute(q)
+        pandas.io.sql.execute(q, conn)
 
     def read_query(self, conn, q):
         """
@@ -355,17 +310,57 @@ class DBModel:
         :return: query results as table
         """
         if isinstance(q, data_algebra.data_ops.ViewRepresentation):
-            q = q.to_sql(db_model=self)
+            temp_tables = dict()
+            q = q.to_sql(db_model=self, temp_tables=temp_tables)
+            if len(temp_tables) > 1:
+                raise ValueError("ops require management of temp tables, please collect them via to_sql(temp_tables)")
         else:
             q = str(q)
-        cur = conn.cursor()
-        cur.execute(q)
-        r = cur.fetchall()
-        colnames = [desc[0] for desc in cur.description]
-        r = self.local_data_model.pd.DataFrame(columns=colnames, data=r)
+        r = pandas.io.sql.read_sql(q, conn)
+        r = self.local_data_model.pd.DataFrame(r)
         r = r.reset_index(drop=True)
         return r
 
+    def table_exists(self, conn, table_name):
+        q_table_name = self.quote_table_name(table_name)
+        # noinspection PyBroadException
+        table_exists = True
+        try:
+            self.read_query(conn, "SELECT * FROM " + q_table_name + " LIMIT 1")
+        except Exception:
+            table_exists = False
+        return table_exists
+
+    def drop_table(self, conn, table_name, *, check=True):
+        if (not check) or self.table_exists(conn, table_name):
+            q_table_name = self.quote_table_name(table_name)
+            self.execute(conn, self.drop_text + ' ' + q_table_name)
+
+    # noinspection PyMethodMayBeStatic,SqlNoDataSourceInspection
+    def insert_table(
+        self, conn, d, table_name, *, qualifiers=None, allow_overwrite=False
+    ):
+        """
+        Insert a table.
+
+        :param conn: a database connection
+        :param d: a Pandas table
+        :param table_name: name to give write to
+        :param qualifiers: schema and such
+        :param allow_overwrite logical, if True drop previous table
+        """
+
+        if qualifiers is not None:
+            raise ValueError("non-empty qualifiers not yet supported on insert")
+        if self.table_exists(conn, table_name):
+            if not allow_overwrite:
+                raise ValueError("table " + table_name + " already exists")
+            else:
+                self.drop_table(conn, table_name, check=False)
+        # Note: the Pandas to_sql() method appears to have SQLite hard-wired into it
+        # it refers to sqlite_master
+        d.to_sql(name=table_name, con=conn, index=False)
+
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
     def read_table(self, conn, table_name, *, qualifiers=None, limit=None):
         if not isinstance(table_name, str):
@@ -391,32 +386,36 @@ class DBModel:
         if not isinstance(identifier, str):
             raise TypeError("expected identifier to be a str")
         if self.identifier_quote in identifier:
-            # TODO: escape quote
-            # TODO: unify with derived PostgreSQL and BigQuery
-            raise TypeError('did not expect ' + self.identifier_quote + ' in identifier')
+            raise ValueError('did not expect ' + self.identifier_quote + ' in identifier')
         return self.identifier_quote + identifier + self.identifier_quote
 
     def quote_table_name(self, table_description):
-        if isinstance(table_description, str):
-            return self.quote_identifier(table_description)
-        if table_description.node_name != "TableDescription":
-            raise TypeError(
-                "Expected table_description to be a data_algebra.data_ops.TableDescription)"
-            )
-        return self.quote_identifier(table_description.table_name)
+        if not isinstance(table_description, str):
+            try:
+                if table_description.node_name == "TableDescription":
+                    table_description = table_description.table_name
+                else:
+                    raise TypeError(
+                        "Expected table_description to be a string or data_algebra.data_ops.TableDescription)"
+                    )
+            except KeyError:
+                raise TypeError(
+                    "Expected table_description to be a string or data_algebra.data_ops.TableDescription)"
+                )
+        return self.quote_identifier(table_description)
 
     def quote_string(self, string):
         if not isinstance(string, str):
             raise TypeError("expected string to be a str")
-        # replace all single-quotes with doubled single quotes and return surrounded by single quotes
+        # replace all string with doubled string quotes
         return (
             self.string_quote
             + re.sub(self.string_quote, self.string_quote + self.string_quote, string)
             + self.string_quote
         )
 
-    def quote_literal(self, string):
-        return self.quote_string(str(string))
+    def quote_literal(self, val):
+        return self.quote_string(str(val))
 
     def value_to_sql(self, v):
         if v is None:
@@ -687,8 +686,10 @@ class DBModel:
             )
         if temp_id_source is None:
             temp_id_source = [0]
+        using_was_None = False
         if using is None:
             using = order_node.column_set
+            using_was_None = True
         subusing = order_node.columns_used_from_sources(using=using)[0]
         subusing = [c for c in order_node.column_names if c in subusing]  # fix order
         subsql = order_node.sources[0].to_near_sql_implementation(
@@ -696,7 +697,9 @@ class DBModel:
         )
         view_name = "order_rows_" + str(temp_id_source[0])
         temp_id_source[0] = temp_id_source[0] + 1
-        terms = {ci: None for ci in subusing}
+        terms = None
+        if not using_was_None:
+            terms = {ci: None for ci in subusing}
         suffix = ""
         if len(order_node.order_columns) > 0:
             suffix = (
@@ -832,9 +835,9 @@ class DBModel:
         near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql1=sql_left.to_bound_near_sql(columns=using_left),
+            sub_sql1=sql_left.to_bound_near_sql(columns=using_left, force_sql=False),
             joiner=jointype + " JOIN",
-            sub_sql2=sql_right.to_bound_near_sql(columns=using_right),
+            sub_sql2=sql_right.to_bound_near_sql(columns=using_right, force_sql=False),
             suffix=on_terms,
             temp_tables=temp_tables,
             annotation=join_node.to_python_implementation(print_sources=False, indent=-1)
@@ -873,10 +876,10 @@ class DBModel:
         constants_right = None
         if concat_node.id_column is not None:
             constants_left = {
-                concat_node.id_column: self.quote_literal(concat_node.a_name)
+                concat_node.id_column: self.quote_string(concat_node.a_name)
             }
             constants_right = {
-                concat_node.id_column: self.quote_literal(concat_node.b_name)
+                concat_node.id_column: self.quote_string(concat_node.b_name)
             }
             terms.update({concat_node.id_column: None})
         confused_temps = set(sql_left.temp_tables.keys()).intersection(
@@ -1094,7 +1097,7 @@ class DBModel:
             v = terms[k]
         except KeyError:
             pass
-        if v is None:
+        if (v is None) or (v == k):
             return self.quote_identifier(k)
         return v + " AS " + self.quote_identifier(k)
 
@@ -1127,6 +1130,12 @@ class DBModel:
             )
         return sql
 
+    def nearsqlcte_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+        assert isinstance(near_sql, data_algebra.near_sql.NearSQLCommonTableExpression)
+        if force_sql:
+            return "SELECT * FROM " + near_sql.quoted_query_name
+        return near_sql.quoted_query_name
+
     def nearsqltable_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLTable)
         if columns is None:
@@ -1148,14 +1157,19 @@ class DBModel:
 
     def nearsqlunary_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLUnaryStep)
-        if columns is None:
-            columns = [k for k in near_sql.terms.keys()]
+        terms_strs = ['*']  # allow * notation if nothing is specified
         terms = near_sql.terms
-        if (constants is not None) and (len(constants) > 0):
-            terms.update(constants)
-        terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
-        if len(terms_strs) < 1:
-            terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+        if terms is not None:
+            if columns is None:
+                columns = [k for k in terms.keys()]
+            terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
+            if (constants is not None) and (len(constants) > 0):
+                terms_strs = terms_strs + [
+                    v + " AS " + self.quote_identifier(k)
+                    for (k, v) in constants.items()
+                ]
+            if len(terms_strs) < 1:
+                terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
         sql = "SELECT "
         if annotate and (near_sql.annotation is not None) and (len(near_sql.annotation) > 0):
             sql = sql + " -- " + _clean_annotation(near_sql.annotation) + "\n "
@@ -1164,7 +1178,8 @@ class DBModel:
             sql = sql + " " + near_sql.suffix
         return sql
 
-    def nearsqlbinary_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
+    def nearsqlbinary_to_sql_(self, near_sql, *,
+                              columns=None, force_sql=False, constants=None, annotate=False, quoted_query_name=None):
         assert isinstance(near_sql, data_algebra.near_sql.NearSQLBinaryStep)
         if columns is None:
             columns = [k for k in near_sql.terms.keys()]
@@ -1174,18 +1189,27 @@ class DBModel:
         terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
         if len(terms_strs) < 1:
             terms_strs = [f'1 AS {self.quote_identifier("data_algebra_placeholder_col_name")}']
+        is_union = 'union' in near_sql.joiner.lower()
         sql = "SELECT "
         if annotate and (near_sql.annotation is not None) and (len(near_sql.annotation) > 0):
             sql = sql + " -- " + _clean_annotation(near_sql.annotation) + "\n "
+        if is_union:
+            substr_1 = near_sql.sub_sql1.to_sql(db_model=self, annotate=annotate)
+            substr_2 = near_sql.sub_sql2.to_sql(db_model=self, annotate=annotate)
+        else:
+            substr_1 = near_sql.sub_sql1.convert_subsql(db_model=self, annotate=annotate)
+            substr_2 = near_sql.sub_sql2.convert_subsql(db_model=self, annotate=annotate)
         sql = sql + (
                 ", ".join(terms_strs) + " FROM " + " ( "
-                + near_sql.sub_sql1.convert_subsql(db_model=self, annotate=annotate)
+                + substr_1
                 + " " + near_sql.joiner + " "
-                + near_sql.sub_sql2.convert_subsql(db_model=self, annotate=annotate)
+                + substr_2
                 )
         if (near_sql.suffix is not None) and (len(near_sql.suffix) > 0):
             sql = sql + " " + near_sql.suffix
         sql = sql + " ) "
+        if is_union and (quoted_query_name is not None):
+            sql = sql + quoted_query_name + " "
         return sql
 
     def nearsqlq_to_sql_(self, near_sql, *, columns=None, force_sql=False, constants=None, annotate=False):
@@ -1244,10 +1268,6 @@ class DBHandle(data_algebra.eval_model.EvalModel):
         self.close()
 
     def read_query(self, q):
-        if isinstance(q, data_algebra.data_ops.ViewRepresentation):
-            q = q.to_sql(self.db_model)
-        else:
-            q = str(q)
         return self.db_model.read_query(conn=self.conn, q=q)
 
     def describe_table(self, table_name, *, qualifiers=None, row_limit=7):
@@ -1291,19 +1311,18 @@ class DBHandle(data_algebra.eval_model.EvalModel):
                 )
         return self.db_model.read_table(self.conn, handle)
 
+    def execute(self, q):
+        self.db_model.execute(conn=self.conn, q=q)
+
+    def drop_table(self, table_name):
+        self.db_model.drop_table(self.conn, table_name)
+
     def insert_table(self, d, *, table_name, allow_overwrite=False):
         self.db_model.insert_table(
             conn=self.conn, d=d, table_name=table_name, allow_overwrite=allow_overwrite
         )
         return self.describe_table(table_name)
 
-    def execute(self, q):
-        if isinstance(q, data_algebra.data_ops.ViewRepresentation):
-            q = q.to_sql(self.db_model)
-        else:
-            q = str(q)
-        self.db_model.execute(conn=self.conn, q=q)
-
     def to_sql(self, ops,
                *,
                pretty=False,
@@ -1320,9 +1339,9 @@ class DBHandle(data_algebra.eval_model.EvalModel):
                           temp_tables=temp_tables,
                           use_with=use_with)
 
-    def drop_table(self, table_name):
-        q_table_name = self.db_model.quote_table_name(table_name)
-        self.execute(f'DROP TABLE IF EXISTS {q_table_name}')
+    def query_to_csv(self, q, *, res_name):
+        d = self.read_query(q)
+        d.to_csv(res_name, index=False)
 
     def managed_eval(self, ops, *, data_map=None, result_name=None, narrow=True):
         query = ops.to_sql(self.db_model)
@@ -1354,13 +1373,12 @@ class DBHandle(data_algebra.eval_model.EvalModel):
         q_table_name = self.db_model.quote_table_name(result_name)
         drop_query = self.db_model.drop_text + ' ' + q_table_name
         create_query = "CREATE TABLE " + q_table_name + " AS " + query
-        cur = self.conn.cursor()
         # noinspection PyBroadException
         try:
-            cur.execute(drop_query)
+            self.db_model.execute(self.conn, drop_query)
         except Exception:
             pass
-        cur.execute(create_query)
+        self.db_model.execute(self.conn, create_query)
         res = self.describe_table(result_name)
         if data_map is not None:
             data_map[result_name] = res
@@ -1382,5 +1400,8 @@ class DBHandle(data_algebra.eval_model.EvalModel):
 
     def close(self):
         if self.conn is not None:
-            self.conn.close()
+            try:
+                self.conn.close()
+            except Exception:
+                pass
             self.conn = None
