@@ -622,7 +622,7 @@ def _enc_value(value):
 class ColumnReference(Term):
     """class to represent referring to a column"""
 
-    view: any  # typically a ViewReference
+    view: any
     column_name: str
 
     def __init__(self, view, column_name):
@@ -755,7 +755,7 @@ class Expression(Term):
             if isinstance(self.args[0], ColumnReference):
                 return subs[0] + "." + self.op + "()"
             else:
-                return subs[0] + "." + self.op + "()"
+                return "(" + subs[0] + ")." + self.op + "()"
         if len(subs) == 2 and self.inline:
             if want_inline_parens:
                 return "(" + subs[0] + " " + self.op + " " + subs[1] + ")"
