@@ -49,6 +49,15 @@ class FnTerm(PreTerm):
         for vi in self.args:
             columns_seen.add(str(vi))
 
+    def get_views(self):
+        views = list()
+        for ai in self.args:
+            vi = ai.get_views()
+            for vii in vi:
+                if vi not in views:  # expectation is views is size zero or 1
+                    views.append(vii)
+        return views
+
     def replace_view(self, view):
         self.args = [ai.replace_view(view) for ai in self.args]
         return self
